﻿using ELTE.TravelAgency.Models;
using System;
using System.Linq;
using System.Web.Mvc;

namespace ELTE.TravelAgency.Controllers
{
    /// <summary>
    /// Vezérlő típusa
    /// </summary>
    public class HomeController : Controller
    {
        private TravelAgencyEntities _entities;

        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        public HomeController() 
        {
            _entities = new TravelAgencyEntities();

            ViewBag.Cities = _entities.City.ToArray(); // a városokat berakjuk egy tömbbe
        }

        /// <summary>
        /// Épületek listázása.
        /// </summary>
        /// <returns>Az épületek listájának nézete.</returns>
        public ActionResult Index()
        {
            return View("Index", _entities.Building.Include("City").ToList());
        }

        /// <summary>
        /// Épületek listázása.
        /// </summary>
        /// <param name="cityId">Város azonosítója.</param>
        /// <returns>Az épületek listájának nézete.</returns>
        public ActionResult List(Int32? cityId)
        {
            // ha hibás az azonosító, vagy nem adták meg
            if (cityId == null || !_entities.City.Any(c => c.Id == cityId))
                return RedirectToAction("Index"); // átirányítjuk a kezdőoldalra

            // megkeressük a megfelelő város szonosítókat 
            return View("Index", _entities.Building.Include("City").Where(b => b.CityId == cityId).ToList());
        }

        /// <summary>
        /// Épület részleteinek nézete.
        /// </summary>
        /// <param name="buildingId">Épület azonosítója.</param>
        /// <returns>Az épület részletes nézete.</returns>
        public ActionResult Details(Int32? buildingId)
        {
            if (buildingId == null) // nem adtak meg azonosítót
                return RedirectToAction("Index");

            Building building = _entities.Building.Include("City").Include("Apartment").FirstOrDefault(b => b.Id == buildingId);

            if (building == null)
                return HttpNotFound(); // átirányítjuk a nem talált oldalra

            ViewBag.Title = "Épület részletei: " + building.Name + " (" + building.City.Name + ")"; // az oldal címe
            ViewBag.Images = _entities.BuildingImage.Where(image => image.BuildingId == buildingId).Select(image => image.Id).ToList(); // az épülethez tarzozó képek azonosítói

            return View("Details", building);
        }

        /// <summary>
        /// Épület főképének lekérdezése.
        /// </summary>
        /// <param name="buildingId">Épület azonosítója.</param>
        /// <returns>Az épület képe, vagy az alapértelmezett kép.</returns>
        public FileResult ImageForBuilding(Int32? buildingId) 
        {
            if (buildingId == null) // nem adtak meg azonosítót
                return File("~/Content/NoImage.png", "image/png");

            // lekérjük az épület első tárolt képjét (kicsiben)
            Byte[] imageContent = _entities.BuildingImage.Where(image => image.BuildingId == buildingId).Select(image => image.ImageSmall).FirstOrDefault();

            if (imageContent == null) // amennyiben nem sikerült betölteni, egy alapértelmezett képet adunk vissza
                return File("~/Content/NoImage.png", "image/png");

            return File(imageContent, "image/png");
        }

        /// <summary>
        /// Épület egyik képének lekérdezése.
        /// </summary>
        /// <param name="imageId">Kép azonosítója.</param>
        /// <param name="large">Nagy méretű kép lekérése.</param>
        /// <returns>Az épület egy képe, vagy az alapértelmezett kép.</returns>
        public FileResult Image(Int32? imageId, Boolean large = false) 
        {
            if (imageId == null) // nem adtak meg azonosítót
                return File("~/Content/NoImage.png", "image/png");

            // lekérjük a megadott azonosítóval rendelkező képet
            Byte[] imageContent = _entities.BuildingImage.Where(image => image.Id == imageId).Select(image => large ? image.ImageLarge : image.ImageSmall).FirstOrDefault();

            if (imageContent == null) // amennyiben nem sikerült betölteni, egy alapértelmezett képet adunk vissza
                return File("~/Content/NoImage.png", "image/png");

            return File(imageContent, "image/png");
        } 
	}
}